/* ppmio.c - read and write ppm files */

#include <stdio.h>
#include <stdlib.h>
#include "ppmio.h"



#define		YES		-1
#define		NO		0


#define		UCHAR		unsigned char




int ppm_read( char *fname, UCHAR **r, UCHAR **g, UCHAR **b, int *nr, int *nc )
{
FILE *fp;
char magic[100];
int max;
UCHAR *rp, *gp, *bp;
int i, n;
int pix1, pix2, pix3;

	if ( (fp = fopen( fname, "r" )) == NULL ) return YES;

	fscanf( fp, "%s", magic );
	if ( ! strcmp( "P6", magic ) && ! strcmp( "P3", magic ) ) return YES;

	fscanf( fp, "%d %d", nc, nr );
	fscanf( fp, "%d", &max );
	fgetc( fp );	/* read white space character after max pixel value */

	n = *nr * *nc;

	rp = *r = (UCHAR *)malloc( n * sizeof( UCHAR ) );
	gp = *g = (UCHAR *)malloc( n * sizeof( UCHAR ) );
	bp = *b = (UCHAR *)malloc( n * sizeof( UCHAR ) );

	if ( strcmp( "P6", magic ) == 0 ) {
		for( i = 0; i < n; i++ ) {
			*rp++ = fgetc( fp );
			*gp++ = fgetc( fp );
			*bp++ = fgetc( fp );
		}
	} else {
		for( i = 0; i < n; i++ ) {
			fscanf( fp, "%d %d %d", &pix1, &pix2, &pix3 );
			*rp++ = pix1;
			*gp++ = pix2;
			*bp++ = pix3;
		}
	}

	fclose( fp );
	return NO;
}




int ppm_write( char *fname, UCHAR *r, UCHAR *g, UCHAR *b, int nr, int nc )
{
FILE *fp;
int i, n;

	if ( (fp = fopen( fname, "w" )) == NULL ) return YES;

	fprintf( fp, "P6\n%d %d\n255\n", nc, nr );

	n = nr * nc;

	for( i = 0; i < n; i++ ) {
		putc( *r++, fp );
		putc( *g++, fp );
		putc( *b++, fp );
	}

	fclose( fp );
	return NO;
}



int pgm_read( char *fname, UCHAR **im, int *nr, int *nc )
{
FILE *fp;
char magic[100];
int max;
int i, n;

	if ( (fp = fopen( fname, "r" )) == NULL ) return YES;

	fscanf( fp, "%s", magic );
	if ( strcmp( "P5", magic ) ) return YES;

	fscanf( fp, "%d %d", nc, nr );
	fscanf( fp, "%d", &max );
	fgetc( fp );	/* read white space character after max pixel value */

	n = *nr * *nc;

	*im = (UCHAR *)malloc( n * sizeof( UCHAR ) );

	if ( fread( *im, sizeof( UCHAR ), n, fp ) != n ) return YES;

	fclose( fp );
	return NO;
}




int pgm_write( char *fname, UCHAR *im, int nr, int nc )
{
FILE *fp;
int i, n;

	if ( (fp = fopen( fname, "w" )) == NULL ) return YES;

	fprintf( fp, "P5\n%d %d\n255\n", nc, nr );

	n = nr * nc;

	if ( fwrite( im, sizeof( UCHAR ), n, fp ) != n ) return YES;

	fclose( fp );
	return NO;
}

