/* preprocess.c - preprocess an image for gait recognition */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "ppmio.h"


#define		YES			-1
#define		NO			0

#define		UCHAR			unsigned char

/* functin prototypes */
void RGBtoY( UCHAR *rd, UCHAR *gr, UCHAR *bl, UCHAR **out, int nr, int nc );


#define		OUTNR		160
#define		OUTNC		320
#define		TOP		100



void main( int argc, char **argv )
{
int nr, nc;
UCHAR *rd, *gr, *bl;
UCHAR *grey;
int i, j;
int ii, jj;
int sum;
UCHAR *output;
int offset, offset2;

	if ( argc != 3 ) {
		fprintf( stderr, "Usage: %s <input.rgb> <output.pgm>\n", argv[0] );
		exit( 0 );
	}

	if ( ppm_read( argv[1], &rd, &gr, &bl, &nr, &nc ) ) {
		fprintf( stderr, "Error:  failed to read ppm file <%s>\n", argv[1] );
		exit( 0 );
	}
	if ( nr != 480 && nc != 640 ) {
		fprintf( stderr, "Error:  incorrect image dimensions\n" );
		exit( 0 );
	}

	RGBtoY( rd, gr, bl, &grey, nr, nc );

	offset = 0;
	while ( grey[ offset * nc ] < 5 && offset < 100 ) offset++;
	offset2 = 0;
	while ( grey[ offset2 * nc + 3 * nc / 4 ] < 5 && offset2 < 100 ) offset2++;
	if ( offset > 10 ) {
		fprintf( stderr, "Error:  row offset too large (%d)\n", offset );
		exit( 0 );
	}

	fprintf( stderr, "preprocessing %s - offsets: %d,%d\n", argv[2], offset, offset2 );
	if ( offset2 )
		offset += 1;
	if ( offset == 0 && offset2 == 0 ) {
		fprintf( stderr, "Warning:  zero offset in <%s>\n", argv[2] );
		/* offset++; */
	}

	output = (UCHAR *)malloc( OUTNR * OUTNC * sizeof( UCHAR ) );

	for( i = 0; i < OUTNR; i++ ) {
		for( j = 0; j < OUTNC; j++ ) {
			ii = 2 * i + TOP + offset;
			jj = 2 * j;

			sum =	grey[ ii      * nc +  jj     ] +
				grey[(ii + 1) * nc +  jj     ] +
				grey[ ii      * nc + (jj + 1)] +
				grey[(ii + 1) * nc + (jj + 1)];
			output[i * OUTNC + j] = (int)(0.25 * sum + 0.5);

			/*
			sum =	grey[ ii      * nc +  jj     ] +
				grey[ ii      * nc + (jj + 1)];

			output[i * OUTNC + j] = (int)(0.5 * sum + 0.5);
			*/
		}
	}

	if ( pgm_write( argv[2], output, OUTNR, OUTNC ) ) {
		fprintf( stderr, "%s:  write to <%s> failed\n", argv[0], argv[2] );
		exit( 0 );
	}

}



void RGBtoY( UCHAR *rd, UCHAR *gr, UCHAR *bl, UCHAR **out, int nr, int nc )
{
int i, n;

	n = nr * nc;

	if ( (*out = (UCHAR *)malloc( n * sizeof( UCHAR ) )) == NULL ) {
		fprintf( stderr, "Error:  malloc failed in \"RGBtoY\"\n" );
		exit( YES );
	}

	for( i = 0; i < n; i++ )
		(*out)[i] = (UCHAR)(0.299 * rd[i] + 0.587 * gr[i] + 0.114 * bl[i] + 0.5);
}


