/* scalarize.c - extract scalar descriptors from optical flow images */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "ppmio.h"
#include "getargs.h"
#include "nrutil.h"


#define		ABS(x)		(((x) >= 0) ? (x) : -(x))
#define		SQR(x)		((x) * (x))
#define		MAX(x, y)	(((x) > (y)) ? (x) : (y))
#define		MIN(x, y)	(((x) < (y)) ? (x) : (y))


#define		UCHAR		unsigned char

#define		YES		-1
#define		NO		0



/* function prototypes */
void LabelImage( UCHAR *, UCHAR *, int , int , int , int * );
void Centroid( char* vx, char* vy, UCHAR* mask, int nr, int nc, float* xcent, float* ycent );
void WtCentroid( char* vx, char* vy, UCHAR* mask, int nr, int nc, float* xcent, float* ycent );
void XWtCentroid( char* vx, char* vy, UCHAR* mask, int nr, int nc, float* xcent, float* ycent );
void YWtCentroid( char* vx, char* vy, UCHAR* mask, int nr, int nc, float* xcent, float* ycent );
void Blob( char* vx, char* vy, UCHAR* mask, int nr, int nc, float* aspect );
void WtBlob( char* vx, char* vy, UCHAR* mask, int nr, int nc, float* aspect );
void DiddleMask( char *xv, char *yv, UCHAR *mask, int nr, int nc,
	int frame, int start, int stop, int xwid, int x1, int x2, int ymin, int ymax );



int verified = YES;		/* use verified flow */
int start = 0;
int stop = 5;
char *xin = "xout%03d.pgm";
char *yin = "yout%03d.pgm";
char *min = "mask%03d.pgm";
char *mout = "out%03d.pgm";
int ymin = 10;
int ymax = 135;
int xwidth = 40;
int xstart = 0;
int xstop = 5;

ARG arglist[] = {
	{	"start",	INTEGER,	&start,		"start image index"	},
	{	"stop",		INTEGER,	&stop,		"stop image index"	},
	{	"x",		STRING,		(int *)&xin,	"x flow name format"	},
	{	"y",		STRING,		(int *)&yin,	"y flow name format"	},
	{	"mask",		STRING,		(int *)&min,	"mask name format"	},
	{	"out",		STRING,		(int *)&mout,	"output mask name fmt"	},
	{	"v",		BOOLEAN,	&verified,	"use verified flow"	},
	{	"ymin",		INTEGER,	&ymin,		"top of motion area"	},
	{	"ymax",		INTEGER,	&ymax,		"bottom of motion area"	},
	{	"xwidth",	INTEGER,	&xwidth,	"width of motion area"	},
	{	"xstart",	INTEGER,	&xstart,	"start motion postion"	},
	{	"xstop",	INTEGER,	&xstop,		"stop motion position"	},
};

#define		NARGS		(sizeof( arglist ) / sizeof( ARG ))



main( int argc, char **argv )
{
int nr, nc, n, nr2, nc2;
char inname[200], outname[200];
char *xflow, *yflow;
UCHAR *mask;
int ncomponents;
int i, j;
float xcent, ycent, xwcent, ywcent, xcendif, ycendif;
float xxwcent, yxwcent, xywcent, yywcent;
float aspect, waspect, aspdif;

	/* check command line args */
	argc = getargs( argc, argv, arglist, NARGS );
	if ( argc != 1 ) {
		fprintf( stderr, "Usage: %s [switches]\n",
				argv[0] );
		pr_usage( arglist, NARGS );
		exit( 0 );
	}
	if ( stop < start ) {
		fprintf( stderr, "Error:  stop index must be >= start index\n" );
		exit( 0 );
	}

	/* read first image to get image size */
	sprintf( inname, xin, start );
	if ( pgm_read( inname, (UCHAR **)&xflow, &nr, &nc ) ) {
		fprintf( stderr, "%s:  failed to read image <%s>\n", argv[0], inname );
		exit( 0 );
	}
	free( xflow );
	n = nr * nc;

	for( i = start; i < stop; i++ ) {
		/* read in first image and check dimensions */
		sprintf( inname, xin, i );
		if ( pgm_read( inname, (UCHAR **)&xflow, &nr2, &nc2 ) ) {
			fprintf( stderr, "%s:  failed to read image <%s>\n", argv[0], inname );
			exit( 0 );
		}
		if ( nr != nr2 || nc != nc2 ) {
			fprintf( stderr, "%s:  image dimensions do not match\n", argv[0] );
			exit( 0 );
		}

		/* read in second image and make sure that dimensions match */
		sprintf( inname, yin, i );
		if ( pgm_read( inname, (UCHAR **)&yflow, &nr2, &nc2 ) ) {
			fprintf( stderr, "%s:  failed to read image <%s>\n", argv[0], inname );
			exit( 0 );
		}
		if ( nr != nr2 || nc != nc2 ) {
			fprintf( stderr, "%s:  image dimensions do not match\n", argv[0] );
			exit( 0 );
		}

		/* read the verification mask if required */
		if ( verified ) {
			/* get verification mask */
			sprintf( inname, min, i );
			if ( pgm_read( inname, &mask, &nr2, &nc2 ) ) {
				fprintf( stderr, "%s:  failed to read image <%s>\n", argv[0], inname );
				exit( 0 );
			}
			if ( nr != nr2 || nc != nc2 ) {
				fprintf( stderr, "%s:  image dimensions do not match\n", argv[0] );
				exit( 0 );
			}
		} else {
			/* assume flow is verified everywhere */
			mask = (UCHAR *)malloc( n * sizeof( UCHAR ) );
			for( j = 0; j < n; j++ )
				mask[j] = 255;
		}

		fprintf( stderr, "Processing frame %4d\r", i );

		DiddleMask( xflow, yflow, mask, nr, nc, i, start, stop,
				xwidth, xstart, xstop, ymin, ymax );

		Centroid( xflow, yflow, mask, nr, nc, &xcent, &ycent );
		WtCentroid( xflow, yflow, mask, nr, nc, &xwcent, &ywcent );
		xcendif = xcent - xwcent;
		ycendif = ycent - ywcent;
		XWtCentroid( xflow, yflow, mask, nr, nc, &xxwcent, &yxwcent );
		YWtCentroid( xflow, yflow, mask, nr, nc, &xywcent, &yywcent );
		Blob( xflow, yflow, mask, nr, nc, &aspect );
		WtBlob( xflow, yflow, mask, nr, nc, &waspect );
		aspdif = aspect - waspect;

		fprintf( stdout,
	"%15.6f %15.6f %15.6f %15.6f %15.6f %15.6f %15.6f %15.6f %15.6f %15.6f %15.6f %15.6f %15.6f\n",
			xcent, ycent, xwcent, ywcent, xcendif, ycendif,
			aspect, waspect, aspdif,
			xxwcent, yxwcent, xywcent, yywcent );

		sprintf( outname, mout, i );
		pgm_write( outname, mask, nr, nc );

		free( xflow );
		free( yflow );
		free( mask );
	}

	fprintf( stderr, "\n" );
}


void Centroid( char* vx, char* vy, UCHAR* mask, int nr, int nc, float* xcent, float* ycent )
{
int i, j;
float sumx, sumy;
int count;

	sumx = sumy = 0.0;
	count = 0;
	for( i = 0; i < nr; i++ )
		for( j = 0; j < nc; j++ )
			if ( mask[i * nc + j] ) {
				count++;
				sumx += j;
				sumy += i;
			}
	*xcent = sumx / count;
	*ycent = sumy / count;
}


void WtCentroid( char* vx, char* vy, UCHAR* mask, int nr, int nc, float* xcent, float* ycent )
{
int i, j;
float sumx, sumy, w, x, y, total;

	sumx = sumy = total = 0.0;
	for( i = 0; i < nr; i++ )
		for( j = 0; j < nc; j++ )
			if ( mask[i * nc + j] ) {
				x = vx[i * nc + j];
				if ( x >= 128 ) x -= 256;
				y = vy[i * nc + j];
				if ( y >= 128 ) y -= 256;
				w = sqrt( SQR(x) + SQR(y) );

				total += w;
				sumx += w * j;
				sumy += w * i;
			}
	*xcent = sumx / total;
	*ycent = sumy / total;
}


void XWtCentroid( char* vx, char* vy, UCHAR* mask, int nr, int nc, float* xcent, float* ycent )
{
int i, j;
float sumx, sumy, w, x, y, total;

	sumx = sumy = total = 0.0;
	for( i = 0; i < nr; i++ )
		for( j = 0; j < nc; j++ )
			if ( mask[i * nc + j] ) {
				x = vx[i * nc + j];
				if ( x >= 128 ) x -= 256;
				y = vy[i * nc + j];
				if ( y >= 128 ) y -= 256;
				w = ABS(x);

				total += w;
				sumx += w * j;
				sumy += w * i;
			}
	*xcent = sumx / total;
	*ycent = sumy / total;
}


void YWtCentroid( char* vx, char* vy, UCHAR* mask, int nr, int nc, float* xcent, float* ycent )
{
int i, j;
float sumx, sumy, w, x, y, total;

	sumx = sumy = total = 0.0;
	for( i = 0; i < nr; i++ )
		for( j = 0; j < nc; j++ )
			if ( mask[i * nc + j] ) {
				x = vx[i * nc + j];
				if ( x >= 128 ) x -= 256;
				y = vy[i * nc + j];
				if ( y >= 128 ) y -= 256;
				w = ABS(y);

				total += w;
				sumx += w * j;
				sumy += w * i;
			}
	*xcent = sumx / total;
	*ycent = sumy / total;
}




/* minimum number of voxels for useful analysis */
#define		MINVOXELS		5

void Blob( char* vx, char* vy, UCHAR* mask, int nr, int nc, float* aspect )
{
int i, j;				/* loop indices */
int n;					/* number of contraband voxels */
float sumx, sumy;			/* sum of x, y postions */
float sumxx, sumyy;			/* sum of squares of x, y postions */
float sumxy;				/* sum of x, y position products */
float **a, *d, **v;			/* matrices for Karhunen-Loeve transform */
int nrot;				/* number of rotation required to do Karhunen-Loeve tx */


	/* allocate arrays for Karhunen-Loeve tx */
	a = matrix( 1, 2, 1, 2 );
	d = vector( 1, 2 );
	v = matrix( 1, 2, 1, 2 );

	/* count up stats on contraband voxels */
	n = 0;
	sumx = sumy = sumxx = sumyy  = sumxy = 0.0;
	for( i = 0; i < nr; i++ )
		for( j = 0; j < nc; j++ )
			if ( mask[i * nc + j] ) {
				n++;

				sumx += i;
				sumy += j;
				sumxx += i * i;
				sumyy += j * j;
				sumxy += i * j;
			}



	/* if sufficient voxels, do a Karhunen-Loeve transform */
	if ( n > MINVOXELS ) {
		a[1][1] = sumxx / (float)n - sumx * sumx / (float)(n * n);
		a[1][2] = sumxy / (float)n - sumx * sumy / (float)(n * n);

		a[2][1] = a[1][2];
		a[2][2] = sumyy / (float)n - sumy * sumy / (float)(n * n);

		jacobi( a, 2, d, v, &nrot );

		d[1] = sqrt( d[1] );
		d[2] = sqrt( d[2] );
		*aspect = d[1] / d[2];
	} else {
		*aspect = 0.0;
	}


	free_matrix( a, 1, 2, 1, 2 );
	free_vector( d, 1, 2 );
	free_matrix( v, 1, 2, 1, 2 );
}



void WtBlob( char* vx, char* vy, UCHAR* mask, int nr, int nc, float* aspect )
{
int i, j;				/* loop indices */
int n;					/* number of contraband voxels */
float sumx, sumy;			/* sum of x, y postions */
float sumxx, sumyy;			/* sum of squares of x, y postions */
float sumxy;				/* sum of x, y position products */
float w, x, y, total;
float **a, *d, **v;			/* matrices for Karhunen-Loeve transform */
int nrot;				/* number of rotation required to do Karhunen-Loeve tx */


	/* allocate arrays for Karhunen-Loeve tx */
	a = matrix( 1, 2, 1, 2 );
	d = vector( 1, 2 );
	v = matrix( 1, 2, 1, 2 );

	/* count up stats on contraband voxels */
	n = 0;
	sumx = sumy = sumxx = sumyy  = sumxy = 0.0;
	total = 0.0;
	for( i = 0; i < nr; i++ )
		for( j = 0; j < nc; j++ )
			if ( mask[i * nc + j] ) {
				n++;
				x = vx[i * nc + j];
				if ( x >= 128 ) x -= 256;
				y = vy[i * nc + j];
				if ( y >= 128 ) y -= 256;
				w = sqrt( SQR(x) + SQR(y) );

				total += w;

				sumx += w * i;
				sumy += w * j;
				sumxx += w * SQR( i );
				sumyy += w * SQR( j );
				sumxy += w * i * j;
			}



	/* if sufficient voxels, do a Karhunen-Loeve transform */
	if ( n > MINVOXELS ) {
		a[1][1] = sumxx / (float)total - sumx * sumx / (float)(total * total);
		a[1][2] = sumxy / (float)total - sumx * sumy / (float)(total * total);

		a[2][1] = a[1][2];
		a[2][2] = sumyy / (float)total - sumy * sumy / (float)(total * total);

		jacobi( a, 2, d, v, &nrot );

		d[1] = sqrt( d[1] );
		d[2] = sqrt( d[2] );
		*aspect = d[1] / d[2];
	} else {
		*aspect = 0.0;
	}


	free_matrix( a, 1, 2, 1, 2 );
	free_vector( d, 1, 2 );
	free_matrix( v, 1, 2, 1, 2 );
}





void DiddleMask( char *xv, char *yv, UCHAR *mask, int nr, int nc,
	int frame, int start, int stop, int xwid, int x1, int x2, int ymin, int ymax )
{
int i, j, n, count;
UCHAR *components;
int ncomponents;
float sumx, sumxx, sumy, sumyy;
float meanx, meany, stdvx, stdvy;
float dx, dy;
int xc;

	n = nr * nc;
	components = (UCHAR *)malloc( n * sizeof( UCHAR ) );

	xc = ((float)(frame - start)/(float)(stop - start)) * (x2 - x1) + x1 + 0.5;
	for( i = 0; i < nr; i++ )
		for( j = 0; j < nc; j++ ) {
			if ( i <= ymin ) mask[i * nc + j] = 0;
			if ( i >= ymax ) mask[i * nc + j] = 0;
			if ( ABS( j - xc ) >= xwid ) mask[i * nc + j] = 0;
		}

	for( i = 0; i < n; i++ )
		if ( (xv[i] || yv[i]) && mask[i] ) mask[i] = 255;
			else mask[i] = 0;

	LabelImage( mask, components, nr, nc, 128, &ncomponents );

	for( j = 1; j <= ncomponents; j++ ) {
		count = 0;
		for( i = 0; i < n; i++ )
			if ( components[i] == j ) count++;

		if ( count < 10 )
			for( i = 0; i < n; i++ )
				if ( components[i] == j ) mask[i] = 0;
	}

	/*
	sumx = sumxx = sumy = sumyy = 0.0;
	count = 0;
	for( i = 0; i < nr; i++ )
		for( j = 0; j < nc; j++ )
			if ( mask[i * nc + j] ) {
				sumx += j;
				sumxx += SQR(j);
				sumy += i;
				sumyy += SQR(i);
				count++;
			}

	meanx = sumx / count;
	meany = sumy / count;
	stdvx = sqrt( sumxx / count - SQR( meanx ) );
	stdvy = sqrt( sumyy / count - SQR( meany ) );

	for( i = 0; i < nr; i++ )
		for( j = 0; j < nc; j++ ) {
			dx = ABS( j - meanx );
			dy = ABS( i - meany );
			if ( (dx > 2.0 * stdvx) || (dy > 2.0 * stdvy) ) mask[i * nc + j] = 0;
		}

	*/

	free( components );
}
